@@ -8,21 +8,21 @@
 
 static sqfs_u8 buffer[4096];
 
-int write_data_from_file(const char *filename, sqfs_data_writer_t *data,
-			 sqfs_inode_generic_t *inode, sqfs_file_t *file,
+int write_data_from_file(const char *filename, sqfs_block_processor_t *data,
+			 sqfs_inode_generic_t **inode, sqfs_file_t *file,
 			 int flags)
 {
 	sqfs_u64 filesz, offset;
 	size_t diff;
 	int ret;
 
-	ret = sqfs_data_writer_begin_file(data, inode, flags);
+	ret = sqfs_block_processor_begin_file(data, inode, flags);
 	if (ret) {
 		sqfs_perror(filename, "beginning file data blocks", ret);
 		return -1;
 	}
 
-	sqfs_inode_get_file_size(inode, &filesz);
+	filesz = file->get_size(file);
 
 	for (offset = 0; offset < filesz; offset += diff) {
 		if (filesz - offset > sizeof(buffer)) {
@@ -37,14 +37,14 @@ int write_data_from_file(const char *filename, sqfs_data_writer_t *data,
 			return -1;
 		}
 
-		ret = sqfs_data_writer_append(data, buffer, diff);
+		ret = sqfs_block_processor_append(data, buffer, diff);
 		if (ret) {
 			sqfs_perror(filename, "packing file data", ret);
 			return -1;
 		}
 	}
 
-	ret = sqfs_data_writer_end_file(data);
+	ret = sqfs_block_processor_end_file(data);
 	if (ret) {
 		sqfs_perror(filename, "finishing file data", ret);
 		return -1;
