@@ -85,19 +85,19 @@ int sqfs_writer_init(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *wrcfg)
 	if (fstree_init(&sqfs->fs, wrcfg->fs_defaults))
 		goto fail_file;
 
-	sqfs->cmp = sqfs_compressor_create(&cfg);
+	ret = sqfs_compressor_create(&cfg, &sqfs->cmp);
 
 #ifdef WITH_LZO
 	if (cfg.id == SQFS_COMP_LZO) {
 		if (sqfs->cmp != NULL)
-			sqfs->cmp->destroy(sqfs->cmp);
+			sqfs_destroy(sqfs->cmp);
 
-		sqfs->cmp = lzo_compressor_create(&cfg);
+		ret = lzo_compressor_create(&cfg, &sqfs->cmp);
 	}
 #endif
 
-	if (sqfs->cmp == NULL) {
-		fputs("Error creating compressor\n", stderr);
+	if (ret != 0) {
+		sqfs_perror(wrcfg->filename, "creating compressor", ret);
 		goto fail_fs;
 	}
 
@@ -123,20 +123,29 @@ int sqfs_writer_init(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *wrcfg)
 	if (ret > 0)
 		sqfs->super.flags |= SQFS_FLAG_COMPRESSOR_OPTIONS;
 
-	sqfs->data = sqfs_data_writer_create(sqfs->super.block_size,
-					     sqfs->cmp, wrcfg->num_jobs,
-					     wrcfg->max_backlog,
-					     wrcfg->devblksize,
-					     sqfs->outfile);
-	if (sqfs->data == NULL) {
-		perror("creating data block processor");
+	sqfs->blkwr = sqfs_block_writer_create(sqfs->outfile,
+					       wrcfg->devblksize, 0);
+	if (sqfs->blkwr == NULL) {
+		perror("creating block writer");
 		goto fail_cmp;
 	}
 
-	memset(&sqfs->stats, 0, sizeof(sqfs->stats));
-	register_stat_hooks(sqfs->data, &sqfs->stats);
+	sqfs->fragtbl = sqfs_frag_table_create(0);
+	if (sqfs->fragtbl == NULL) {
+		perror("creating fragment table");
+		goto fail_blkwr;
+	}
 
-	sqfs->idtbl = sqfs_id_table_create();
+	sqfs->data = sqfs_block_processor_create(sqfs->super.block_size,
+						 sqfs->cmp, wrcfg->num_jobs,
+						 wrcfg->max_backlog,
+						 sqfs->blkwr, sqfs->fragtbl);
+	if (sqfs->data == NULL) {
+		perror("creating data block processor");
+		goto fail_fragtbl;
+	}
+
+	sqfs->idtbl = sqfs_id_table_create(0);
 	if (sqfs->idtbl == NULL) {
 		sqfs_perror(wrcfg->filename, "creating ID table",
 			    SQFS_ERROR_ALLOC);
@@ -178,22 +187,26 @@ int sqfs_writer_init(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *wrcfg)
 
 	return 0;
 fail_dm:
-	sqfs_meta_writer_destroy(sqfs->dm);
+	sqfs_destroy(sqfs->dm);
 fail_im:
-	sqfs_meta_writer_destroy(sqfs->im);
+	sqfs_destroy(sqfs->im);
 fail_xwr:
 	if (sqfs->xwr != NULL)
-		sqfs_xattr_writer_destroy(sqfs->xwr);
+		sqfs_destroy(sqfs->xwr);
 fail_id:
-	sqfs_id_table_destroy(sqfs->idtbl);
+	sqfs_destroy(sqfs->idtbl);
 fail_data:
-	sqfs_data_writer_destroy(sqfs->data);
+	sqfs_destroy(sqfs->data);
+fail_fragtbl:
+	sqfs_destroy(sqfs->fragtbl);
+fail_blkwr:
+	sqfs_destroy(sqfs->blkwr);
 fail_cmp:
-	sqfs->cmp->destroy(sqfs->cmp);
+	sqfs_destroy(sqfs->cmp);
 fail_fs:
 	fstree_cleanup(&sqfs->fs);
 fail_file:
-	sqfs->outfile->destroy(sqfs->outfile);
+	sqfs_destroy(sqfs->outfile);
 	return -1;
 }
 
@@ -204,7 +217,7 @@ int sqfs_writer_finish(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *cfg)
 	if (!cfg->quiet)
 		fputs("Waiting for remaining data blocks...\n", stdout);
 
-	ret = sqfs_data_writer_finish(sqfs->data);
+	ret = sqfs_block_processor_finish(sqfs->data);
 	if (ret) {
 		sqfs_perror(cfg->filename, "finishing data blocks", ret);
 		return -1;
@@ -221,7 +234,8 @@ int sqfs_writer_finish(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *cfg)
 	if (!cfg->quiet)
 		fputs("Writing fragment table...\n", stdout);
 
-	ret = sqfs_data_writer_write_fragment_table(sqfs->data, &sqfs->super);
+	ret = sqfs_frag_table_write(sqfs->fragtbl, sqfs->outfile,
+				    &sqfs->super, sqfs->cmp);
 	if (ret) {
 		sqfs_perror(cfg->filename, "writing fragment table", ret);
 		return -1;
@@ -277,7 +291,7 @@ int sqfs_writer_finish(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *cfg)
 	}
 
 	if (!cfg->quiet)
-		sqfs_print_statistics(&sqfs->super, &sqfs->stats);
+		sqfs_print_statistics(&sqfs->super, sqfs->data, sqfs->blkwr);
 
 	return 0;
 }
@@ -285,14 +299,16 @@ int sqfs_writer_finish(sqfs_writer_t *sqfs, const sqfs_writer_cfg_t *cfg)
 void sqfs_writer_cleanup(sqfs_writer_t *sqfs)
 {
 	if (sqfs->xwr != NULL)
-		sqfs_xattr_writer_destroy(sqfs->xwr);
+		sqfs_destroy(sqfs->xwr);
 
-	sqfs_dir_writer_destroy(sqfs->dirwr);
-	sqfs_meta_writer_destroy(sqfs->dm);
-	sqfs_meta_writer_destroy(sqfs->im);
-	sqfs_id_table_destroy(sqfs->idtbl);
-	sqfs_data_writer_destroy(sqfs->data);
-	sqfs->cmp->destroy(sqfs->cmp);
+	sqfs_destroy(sqfs->dirwr);
+	sqfs_destroy(sqfs->dm);
+	sqfs_destroy(sqfs->im);
+	sqfs_destroy(sqfs->idtbl);
+	sqfs_destroy(sqfs->data);
+	sqfs_destroy(sqfs->blkwr);
+	sqfs_destroy(sqfs->fragtbl);
+	sqfs_destroy(sqfs->cmp);
 	fstree_cleanup(&sqfs->fs);
-	sqfs->outfile->destroy(sqfs->outfile);
+	sqfs_destroy(sqfs->outfile);
 }
