@@ -12,8 +12,8 @@
 
 #include "internal.h"
 
-typedef sqfs_compressor_t *(*compressor_fun_t)
-	(const sqfs_compressor_config_t *cfg);
+typedef int (*compressor_fun_t)(const sqfs_compressor_config_t *cfg,
+				sqfs_compressor_t **out);
 
 static compressor_fun_t compressors[SQFS_COMP_MAX + 1] = {
 #ifdef WITH_GZIP
@@ -73,25 +73,24 @@ int sqfs_generic_read_options(sqfs_file_t *file, void *data, size_t size)
 	return 0;
 }
 
-bool sqfs_compressor_exists(E_SQFS_COMPRESSOR id)
-{
-	if (id < SQFS_COMP_MIN || id > SQFS_COMP_MAX)
-		return false;
-
-	return (compressors[id] != NULL);
-}
-
-sqfs_compressor_t *sqfs_compressor_create(const sqfs_compressor_config_t *cfg)
+int sqfs_compressor_create(const sqfs_compressor_config_t *cfg,
+			   sqfs_compressor_t **out)
 {
 	sqfs_u8 padd0[sizeof(cfg->opt)];
 	int ret;
 
-	if (cfg == NULL || cfg->id < SQFS_COMP_MIN || cfg->id > SQFS_COMP_MAX)
-		return NULL;
+	/* check compressor ID */
+	if (cfg == NULL)
+		return SQFS_ERROR_ARG_INVALID;
+
+	if (cfg->id < SQFS_COMP_MIN || cfg->id > SQFS_COMP_MAX)
+		return SQFS_ERROR_UNSUPPORTED;
 
 	if (compressors[cfg->id] == NULL)
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 
+	/* make sure the padding bytes are cleared, so we could theoretically
+	   turn them into option fields in the future and remain compatible */
 	memset(padd0, 0, sizeof(padd0));
 
 	switch (cfg->id) {
@@ -117,12 +116,12 @@ sqfs_compressor_t *sqfs_compressor_create(const sqfs_compressor_config_t *cfg)
 	}
 
 	if (ret != 0)
-		return NULL;
+		return SQFS_ERROR_ARG_INVALID;
 
-	return compressors[cfg->id](cfg);
+	return compressors[cfg->id](cfg, out);
 }
 
-const char *sqfs_compressor_name_from_id(E_SQFS_COMPRESSOR id)
+const char *sqfs_compressor_name_from_id(SQFS_COMPRESSOR id)
 {
 	if (id < 0 || (size_t)id >= sizeof(names) / sizeof(names[0]))
 		return NULL;
@@ -143,17 +142,16 @@ int sqfs_compressor_id_from_name(const char *name)
 }
 
 int sqfs_compressor_config_init(sqfs_compressor_config_t *cfg,
-				E_SQFS_COMPRESSOR id,
+				SQFS_COMPRESSOR id,
 				size_t block_size, sqfs_u16 flags)
 {
-	memset(cfg, 0, sizeof(*cfg));
+	sqfs_u32 flag_mask = SQFS_COMP_FLAG_GENERIC_ALL;
 
-	cfg->id = id;
-	cfg->flags = flags;
-	cfg->block_size = block_size;
+	memset(cfg, 0, sizeof(*cfg));
 
 	switch (id) {
 	case SQFS_COMP_GZIP:
+		flag_mask |= SQFS_COMP_FLAG_GZIP_ALL;
 		cfg->opt.gzip.level = SQFS_GZIP_DEFAULT_LEVEL;
 		cfg->opt.gzip.window_size = SQFS_GZIP_DEFAULT_WINDOW;
 		break;
@@ -165,11 +163,25 @@ int sqfs_compressor_config_init(sqfs_compressor_config_t *cfg,
 		cfg->opt.zstd.level = SQFS_ZSTD_DEFAULT_LEVEL;
 		break;
 	case SQFS_COMP_XZ:
+		flag_mask |= SQFS_COMP_FLAG_XZ_ALL;
 		cfg->opt.xz.dict_size = block_size;
 		break;
-	default:
+	case SQFS_COMP_LZMA:
 		break;
+	case SQFS_COMP_LZ4:
+		flag_mask |= SQFS_COMP_FLAG_LZ4_ALL;
+		break;
+	default:
+		return SQFS_ERROR_UNSUPPORTED;
 	}
 
+	if (flags & ~flag_mask) {
+		memset(cfg, 0, sizeof(*cfg));
+		return SQFS_ERROR_UNSUPPORTED;
+	}
+
+	cfg->id = id;
+	cfg->flags = flags;
+	cfg->block_size = block_size;
 	return 0;
 }
