@@ -31,7 +31,7 @@ typedef struct {
 	gzip_options_t opt;
 } gzip_compressor_t;
 
-static void gzip_destroy(sqfs_compressor_t *base)
+static void gzip_destroy(sqfs_object_t *base)
 {
 	gzip_compressor_t *gzip = (gzip_compressor_t *)base;
 
@@ -44,6 +44,22 @@ static void gzip_destroy(sqfs_compressor_t *base)
 	free(gzip);
 }
 
+static void gzip_get_configuration(const sqfs_compressor_t *base,
+				   sqfs_compressor_config_t *cfg)
+{
+	const gzip_compressor_t *gzip = (const gzip_compressor_t *)base;
+
+	memset(cfg, 0, sizeof(*cfg));
+	cfg->id = SQFS_COMP_GZIP;
+	cfg->flags = gzip->opt.strategies;
+	cfg->block_size = gzip->block_size;
+	cfg->opt.gzip.level = gzip->opt.level;
+	cfg->opt.gzip.window_size = gzip->opt.window;
+
+	if (!gzip->compress)
+		cfg->flags |= SQFS_COMP_FLAG_UNCOMPRESS;
+}
+
 static int gzip_write_options(sqfs_compressor_t *base, sqfs_file_t *file)
 {
 	gzip_compressor_t *gzip = (gzip_compressor_t *)base;
@@ -205,7 +221,7 @@ static sqfs_s32 gzip_do_block(sqfs_compressor_t *base, const sqfs_u8 *in,
 	return 0;
 }
 
-static sqfs_compressor_t *gzip_create_copy(sqfs_compressor_t *cmp)
+static sqfs_object_t *gzip_create_copy(const sqfs_object_t *cmp)
 {
 	gzip_compressor_t *gzip = malloc(sizeof(*gzip));
 	int ret;
@@ -228,10 +244,11 @@ static sqfs_compressor_t *gzip_create_copy(sqfs_compressor_t *cmp)
 		return NULL;
 	}
 
-	return (sqfs_compressor_t *)gzip;
+	return (sqfs_object_t *)gzip;
 }
 
-sqfs_compressor_t *gzip_compressor_create(const sqfs_compressor_config_t *cfg)
+int gzip_compressor_create(const sqfs_compressor_config_t *cfg,
+			   sqfs_compressor_t **out)
 {
 	gzip_compressor_t *gzip;
 	sqfs_compressor_t *base;
@@ -239,35 +256,36 @@ sqfs_compressor_t *gzip_compressor_create(const sqfs_compressor_config_t *cfg)
 
 	if (cfg->flags & ~(SQFS_COMP_FLAG_GZIP_ALL |
 			   SQFS_COMP_FLAG_GENERIC_ALL)) {
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 	}
 
 	if (cfg->opt.gzip.level < SQFS_GZIP_MIN_LEVEL ||
 	    cfg->opt.gzip.level > SQFS_GZIP_MAX_LEVEL) {
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 	}
 
 	if (cfg->opt.gzip.window_size < SQFS_GZIP_MIN_WINDOW ||
 	    cfg->opt.gzip.window_size > SQFS_GZIP_MAX_WINDOW) {
-		return NULL;
+		return SQFS_ERROR_UNSUPPORTED;
 	}
 
 	gzip = calloc(1, sizeof(*gzip));
 	base = (sqfs_compressor_t *)gzip;
 
 	if (gzip == NULL)
-		return NULL;
+		return SQFS_ERROR_ALLOC;
 
 	gzip->opt.level = cfg->opt.gzip.level;
 	gzip->opt.window = cfg->opt.gzip.window_size;
 	gzip->opt.strategies = cfg->flags & SQFS_COMP_FLAG_GZIP_ALL;
 	gzip->compress = (cfg->flags & SQFS_COMP_FLAG_UNCOMPRESS) == 0;
 	gzip->block_size = cfg->block_size;
+	base->get_configuration = gzip_get_configuration;
 	base->do_block = gzip_do_block;
-	base->destroy = gzip_destroy;
 	base->write_options = gzip_write_options;
 	base->read_options = gzip_read_options;
-	base->create_copy = gzip_create_copy;
+	((sqfs_object_t *)base)->copy = gzip_create_copy;
+	((sqfs_object_t *)base)->destroy = gzip_destroy;
 
 	if (gzip->compress) {
 		ret = deflateInit2(&gzip->strm, cfg->opt.gzip.level,
@@ -279,8 +297,9 @@ sqfs_compressor_t *gzip_compressor_create(const sqfs_compressor_config_t *cfg)
 
 	if (ret != Z_OK) {
 		free(gzip);
-		return NULL;
+		return SQFS_ERROR_COMPRESSOR;
 	}
 
-	return base;
+	*out = base;
+	return 0;
 }
