@@ -9,6 +9,7 @@
 
 #include "sqfs/data_reader.h"
 #include "sqfs/compressor.h"
+#include "sqfs/frag_table.h"
 #include "sqfs/block.h"
 #include "sqfs/error.h"
 #include "sqfs/table.h"
@@ -20,16 +21,18 @@
 #include <string.h>
 
 struct sqfs_data_reader_t {
-	sqfs_fragment_t *frag;
+	sqfs_object_t obj;
+
+	sqfs_frag_table_t *frag_tbl;
 	sqfs_compressor_t *cmp;
-	sqfs_block_t *data_block;
-	sqfs_block_t *frag_block;
-
-	sqfs_u64 current_block;
-
 	sqfs_file_t *file;
 
-	sqfs_u32 num_fragments;
+	sqfs_u8 *data_block;
+	size_t data_blk_size;
+	sqfs_u64 current_block;
+
+	sqfs_u8 *frag_block;
+	size_t frag_blk_size;
 	sqfs_u32 current_frag_index;
 	sqfs_u32 block_size;
 
@@ -37,100 +40,146 @@ struct sqfs_data_reader_t {
 };
 
 static int get_block(sqfs_data_reader_t *data, sqfs_u64 off, sqfs_u32 size,
-		     size_t unpacked_size, sqfs_block_t **out)
+		     sqfs_u32 max_size, size_t *out_sz, sqfs_u8 **out)
 {
-	sqfs_block_t *blk = alloc_flex(sizeof(*blk), 1, unpacked_size);
 	sqfs_u32 on_disk_size;
 	sqfs_s32 ret;
 	int err;
 
-	if (blk == NULL)
-		return SQFS_ERROR_ALLOC;
+	*out = alloc_array(1, max_size);
+	*out_sz = max_size;
 
-	blk->size = unpacked_size;
-
-	if (SQFS_IS_SPARSE_BLOCK(size)) {
-		*out = blk;
-		return 0;
+	if (*out == NULL) {
+		err = SQFS_ERROR_ALLOC;
+		goto fail;
 	}
 
+	if (SQFS_IS_SPARSE_BLOCK(size))
+		return 0;
+
 	on_disk_size = SQFS_ON_DISK_BLOCK_SIZE(size);
 
-	if (on_disk_size > unpacked_size) {
-		free(blk);
-		return SQFS_ERROR_OVERFLOW;
+	if (on_disk_size > max_size) {
+		err = SQFS_ERROR_OVERFLOW;
+		goto fail;
 	}
 
 	if (SQFS_IS_BLOCK_COMPRESSED(size)) {
 		err = data->file->read_at(data->file, off,
 					  data->scratch, on_disk_size);
-		if (err) {
-			free(blk);
-			return err;
-		}
+		if (err)
+			goto fail;
 
 		ret = data->cmp->do_block(data->cmp, data->scratch,
-					  on_disk_size, blk->data, blk->size);
-		if (ret <= 0)
+					  on_disk_size, *out, max_size);
+		if (ret <= 0) {
 			err = ret < 0 ? ret : SQFS_ERROR_OVERFLOW;
+			goto fail;
+		}
+
+		*out_sz = ret;
 	} else {
 		err = data->file->read_at(data->file, off,
-					  blk->data, on_disk_size);
+					  *out, on_disk_size);
+		if (err)
+			goto fail;
+
+		*out_sz = on_disk_size;
 	}
 
-	if (err) {
-		free(blk);
-		return err;
-	}
-
-	*out = blk;
 	return 0;
+fail:
+	free(*out);
+	*out = NULL;
+	*out_sz = 0;
+	return err;
 }
 
 static int precache_data_block(sqfs_data_reader_t *data, sqfs_u64 location,
 			       sqfs_u32 size)
 {
-	int ret;
-
 	if (data->data_block != NULL && data->current_block == location)
 		return 0;
 
 	free(data->data_block);
-	data->data_block = NULL;
-
-	ret = get_block(data, location, size, data->block_size,
-			&data->data_block);
-
-	if (ret < 0) {
-		data->data_block = NULL;
-		return -1;
-	}
-
 	data->current_block = location;
-	return 0;
+
+	return get_block(data, location, size, data->block_size,
+			 &data->data_blk_size, &data->data_block);
 }
 
 static int precache_fragment_block(sqfs_data_reader_t *data, size_t idx)
 {
+	sqfs_fragment_t ent;
 	int ret;
 
 	if (data->frag_block != NULL && idx == data->current_frag_index)
 		return 0;
 
-	if (idx >= data->num_fragments)
-		return SQFS_ERROR_OUT_OF_BOUNDS;
+	ret = sqfs_frag_table_lookup(data->frag_tbl, idx, &ent);
+	if (ret != 0)
+		return ret;
 
 	free(data->frag_block);
-	data->frag_block = NULL;
-
-	ret = get_block(data, data->frag[idx].start_offset,
-			data->frag[idx].size, data->block_size,
-			&data->frag_block);
-	if (ret < 0)
-		return -1;
-
 	data->current_frag_index = idx;
-	return 0;
+
+	return get_block(data, ent.start_offset, ent.size, data->block_size,
+			 &data->frag_blk_size, &data->frag_block);
+}
+
+static void data_reader_destroy(sqfs_object_t *obj)
+{
+	sqfs_data_reader_t *data = (sqfs_data_reader_t *)obj;
+
+	sqfs_destroy(data->frag_tbl);
+	free(data->data_block);
+	free(data->frag_block);
+	free(data);
+}
+
+static sqfs_object_t *data_reader_copy(const sqfs_object_t *obj)
+{
+	const sqfs_data_reader_t *data = (const sqfs_data_reader_t *)obj;
+	sqfs_data_reader_t *copy;
+
+	copy = alloc_flex(sizeof(*data), 1, data->block_size);
+	if (copy == NULL)
+		return NULL;
+
+	memcpy(copy, data, sizeof(*data) + data->block_size);
+
+	copy->frag_tbl = sqfs_copy(data->frag_tbl);
+	if (copy->frag_tbl == NULL)
+		goto fail_ftbl;
+
+	if (data->data_block != NULL) {
+		copy->data_block = malloc(data->data_blk_size);
+		if (copy->data_block == NULL)
+			goto fail_dblk;
+
+		memcpy(copy->data_block, data->data_block,
+		       data->data_blk_size);
+	}
+
+	if (copy->frag_block != NULL) {
+		copy->frag_block = malloc(copy->frag_blk_size);
+		if (copy->frag_block == NULL)
+			goto fail_fblk;
+
+		memcpy(copy->frag_block, data->frag_block,
+		       data->frag_blk_size);
+	}
+
+	/* XXX: file and cmp aren't deep-copied becaues data
+	        doesn't own them either. */
+	return (sqfs_object_t *)copy;
+fail_fblk:
+	free(copy->data_block);
+fail_dblk:
+	sqfs_destroy(copy->frag_tbl);
+fail_ftbl:
+	free(copy);
+	return NULL;
 }
 
 sqfs_data_reader_t *sqfs_data_reader_create(sqfs_file_t *file,
@@ -139,75 +188,44 @@ sqfs_data_reader_t *sqfs_data_reader_create(sqfs_file_t *file,
 {
 	sqfs_data_reader_t *data = alloc_flex(sizeof(*data), 1, block_size);
 
-	if (data != NULL) {
-		data->file = file;
-		data->block_size = block_size;
-		data->cmp = cmp;
+	if (data == NULL)
+		return NULL;
+
+	data->frag_tbl = sqfs_frag_table_create(0);
+	if (data->frag_tbl == NULL) {
+		free(data);
+		return NULL;
 	}
 
+	((sqfs_object_t *)data)->destroy = data_reader_destroy;
+	((sqfs_object_t *)data)->copy = data_reader_copy;
+	data->file = file;
+	data->block_size = block_size;
+	data->cmp = cmp;
 	return data;
 }
 
 int sqfs_data_reader_load_fragment_table(sqfs_data_reader_t *data,
 					 const sqfs_super_t *super)
 {
-	void *raw_frag;
-	size_t size;
-	sqfs_u32 i;
 	int ret;
 
 	free(data->frag_block);
-	free(data->frag);
-
-	data->frag = NULL;
 	data->frag_block = NULL;
-	data->num_fragments = 0;
 	data->current_frag_index = 0;
 
-	if (super->fragment_entry_count == 0 ||
-	    (super->flags & SQFS_FLAG_NO_FRAGMENTS) != 0) {
-		return 0;
-	}
-
-	if (super->fragment_table_start >= super->bytes_used)
-		return SQFS_ERROR_OUT_OF_BOUNDS;
-
-	if (SZ_MUL_OV(sizeof(data->frag[0]), super->fragment_entry_count,
-		      &size)) {
-		return SQFS_ERROR_OVERFLOW;
-	}
-
-	ret = sqfs_read_table(data->file, data->cmp, size,
-			      super->fragment_table_start,
-			      super->directory_table_start,
-			      super->fragment_table_start, &raw_frag);
-	if (ret)
+	ret = sqfs_frag_table_read(data->frag_tbl, data->file,
+				   super, data->cmp);
+	if (ret != 0)
 		return ret;
 
-	data->num_fragments = super->fragment_entry_count;
-	data->current_frag_index = super->fragment_entry_count;
-	data->frag = raw_frag;
-
-	for (i = 0; i < data->num_fragments; ++i) {
-		data->frag[i].size = le32toh(data->frag[i].size);
-		data->frag[i].start_offset =
-			le64toh(data->frag[i].start_offset);
-	}
-
+	data->current_frag_index = sqfs_frag_table_get_size(data->frag_tbl);
 	return 0;
 }
 
-void sqfs_data_reader_destroy(sqfs_data_reader_t *data)
-{
-	free(data->data_block);
-	free(data->frag_block);
-	free(data->frag);
-	free(data);
-}
-
 int sqfs_data_reader_get_block(sqfs_data_reader_t *data,
 			       const sqfs_inode_generic_t *inode,
-			       size_t index, sqfs_block_t **out)
+			       size_t index, size_t *size, sqfs_u8 **out)
 {
 	size_t i, unpacked_size;
 	sqfs_u64 off, filesz;
@@ -215,52 +233,54 @@ int sqfs_data_reader_get_block(sqfs_data_reader_t *data,
 	sqfs_inode_get_file_block_start(inode, &off);
 	sqfs_inode_get_file_size(inode, &filesz);
 
-	if (index >= inode->num_file_blocks)
+	if (index >= sqfs_inode_get_file_block_count(inode))
 		return SQFS_ERROR_OUT_OF_BOUNDS;
 
 	for (i = 0; i < index; ++i) {
-		off += SQFS_ON_DISK_BLOCK_SIZE(inode->block_sizes[i]);
+		off += SQFS_ON_DISK_BLOCK_SIZE(inode->extra[i]);
 		filesz -= data->block_size;
 	}
 
 	unpacked_size = filesz < data->block_size ? filesz : data->block_size;
 
-	return get_block(data, off, inode->block_sizes[index],
-			 unpacked_size, out);
+	return get_block(data, off, inode->extra[index],
+			 unpacked_size, size, out);
 }
 
 int sqfs_data_reader_get_fragment(sqfs_data_reader_t *data,
 				  const sqfs_inode_generic_t *inode,
-				  sqfs_block_t **out)
+				  size_t *size, sqfs_u8 **out)
 {
 	sqfs_u32 frag_idx, frag_off, frag_sz;
-	sqfs_block_t *blk;
+	size_t block_count;
 	sqfs_u64 filesz;
+	int err;
 
 	sqfs_inode_get_file_size(inode, &filesz);
 	sqfs_inode_get_frag_location(inode, &frag_idx, &frag_off);
+	*size = 0;
+	*out = NULL;
 
-	if (inode->num_file_blocks * data->block_size >= filesz) {
-		*out = NULL;
+	block_count = sqfs_inode_get_file_block_count(inode);
+
+	if (block_count * data->block_size >= filesz)
 		return 0;
-	}
 
 	frag_sz = filesz % data->block_size;
 
-	if (precache_fragment_block(data, frag_idx))
-		return -1;
+	err = precache_fragment_block(data, frag_idx);
+	if (err)
+		return err;
 
 	if (frag_off + frag_sz > data->block_size)
-		return -1;
+		return SQFS_ERROR_OUT_OF_BOUNDS;
 
-	blk = alloc_flex(sizeof(*blk), 1, frag_sz);
-	if (blk == NULL)
-		return -1;
+	*out = alloc_array(1, frag_sz);
+	if (*out == NULL)
+		return SQFS_ERROR_ALLOC;
 
-	blk->size = frag_sz;
-	memcpy(blk->data, (char *)data->frag_block->data + frag_off, frag_sz);
-
-	*out = blk;
+	*size = frag_sz;
+	memcpy(*out, (char *)data->frag_block + frag_off, frag_sz);
 	return 0;
 }
 
@@ -269,9 +289,10 @@ sqfs_s32 sqfs_data_reader_read(sqfs_data_reader_t *data,
 			       sqfs_u64 offset, void *buffer, sqfs_u32 size)
 {
 	sqfs_u32 frag_idx, frag_off, diff, total = 0;
+	size_t i, block_count;
 	sqfs_u64 off, filesz;
 	char *ptr;
-	size_t i;
+	int err;
 
 	if (size >= 0x7FFFFFFF)
 		size = 0x7FFFFFFE;
@@ -280,12 +301,13 @@ sqfs_s32 sqfs_data_reader_read(sqfs_data_reader_t *data,
 	sqfs_inode_get_file_size(inode, &filesz);
 	sqfs_inode_get_frag_location(inode, &frag_idx, &frag_off);
 	sqfs_inode_get_file_block_start(inode, &off);
+	block_count = sqfs_inode_get_file_block_count(inode);
 
 	/* find location of the first block */
 	i = 0;
 
-	while (offset > data->block_size && i < inode->num_file_blocks) {
-		off += SQFS_ON_DISK_BLOCK_SIZE(inode->block_sizes[i++]);
+	while (offset > data->block_size && i < block_count) {
+		off += SQFS_ON_DISK_BLOCK_SIZE(inode->extra[i++]);
 		offset -= data->block_size;
 
 		if (filesz >= data->block_size) {
@@ -296,22 +318,20 @@ sqfs_s32 sqfs_data_reader_read(sqfs_data_reader_t *data,
 	}
 
 	/* copy data from blocks */
-	while (i < inode->num_file_blocks && size > 0 && filesz > 0) {
+	while (i < block_count && size > 0 && filesz > 0) {
 		diff = data->block_size - offset;
 		if (size < diff)
 			diff = size;
 
-		if (SQFS_IS_SPARSE_BLOCK(inode->block_sizes[i])) {
+		if (SQFS_IS_SPARSE_BLOCK(inode->extra[i])) {
 			memset(buffer, 0, diff);
 		} else {
-			if (precache_data_block(data, off,
-						inode->block_sizes[i])) {
-				return -1;
-			}
+			err = precache_data_block(data, off, inode->extra[i]);
+			if (err)
+				return err;
 
-			memcpy(buffer, (char *)data->data_block->data + offset,
-			       diff);
-			off += SQFS_ON_DISK_BLOCK_SIZE(inode->block_sizes[i]);
+			memcpy(buffer, (char *)data->data_block + offset, diff);
+			off += SQFS_ON_DISK_BLOCK_SIZE(inode->extra[i]);
 		}
 
 		if (filesz >= data->block_size) {
@@ -328,9 +348,10 @@ sqfs_s32 sqfs_data_reader_read(sqfs_data_reader_t *data,
 	}
 
 	/* copy from fragment */
-	if (i == inode->num_file_blocks && size > 0 && filesz > 0) {
-		if (precache_fragment_block(data, frag_idx))
-			return -1;
+	if (i == block_count && size > 0 && filesz > 0) {
+		err = precache_fragment_block(data, frag_idx);
+		if (err)
+			return err;
 
 		if (frag_off + filesz > data->block_size)
 			return SQFS_ERROR_OUT_OF_BOUNDS;
@@ -344,7 +365,7 @@ sqfs_s32 sqfs_data_reader_read(sqfs_data_reader_t *data,
 		if (size == 0)
 			return total;
 
-		ptr = (char *)data->frag_block->data + frag_off + offset;
+		ptr = (char *)data->frag_block + frag_off + offset;
 		memcpy(buffer, ptr, size);
 		total += size;
 	}
