@@ -99,6 +99,8 @@ typedef struct kv_block_desc_t {
 } kv_block_desc_t;
 
 struct sqfs_xattr_writer_t {
+	sqfs_object_t base;
+
 	str_table_t keys;
 	str_table_t values;
 
@@ -113,6 +115,81 @@ struct sqfs_xattr_writer_t {
 };
 
 
+static sqfs_object_t *xattr_writer_copy(const sqfs_object_t *obj)
+{
+	const sqfs_xattr_writer_t *xwr = (const sqfs_xattr_writer_t *)obj;
+	kv_block_desc_t *blk, *it, **next;
+	sqfs_xattr_writer_t *copy;
+
+	copy = calloc(1, sizeof(*copy));
+	if (copy == NULL)
+		return NULL;
+
+	memcpy(copy, xwr, sizeof(*xwr));
+
+	if (str_table_copy(&copy->keys, &xwr->keys))
+		goto fail_keys;
+
+	if (str_table_copy(&copy->values, &xwr->values))
+		goto fail_values;
+
+	copy->max_pairs = xwr->num_pairs;
+	copy->num_pairs = xwr->num_pairs;
+
+	copy->kv_pairs = malloc(sizeof(copy->kv_pairs[0]) * xwr->num_pairs);
+	if (copy->kv_pairs == NULL)
+		goto fail_pairs;
+
+	memcpy(copy->kv_pairs, xwr->kv_pairs,
+	       sizeof(copy->kv_pairs[0]) * xwr->num_pairs);
+
+	next = &(copy->kv_blocks);
+
+	for (it = xwr->kv_blocks; it != NULL; it = it->next) {
+		blk = malloc(sizeof(*blk));
+		if (blk == NULL)
+			goto fail_blk;
+
+		memcpy(blk, it, sizeof(*it));
+		blk->next = NULL;
+
+		*next = blk;
+		next = &(blk->next);
+	}
+
+	return (sqfs_object_t *)copy;
+fail_blk:
+	while (copy->kv_blocks != NULL) {
+		blk = copy->kv_blocks;
+		copy->kv_blocks = copy->kv_blocks->next;
+		free(blk);
+	}
+fail_pairs:
+	str_table_cleanup(&copy->values);
+fail_values:
+	str_table_cleanup(&copy->keys);
+fail_keys:
+	free(copy);
+	return NULL;
+}
+
+static void xattr_writer_destroy(sqfs_object_t *obj)
+{
+	sqfs_xattr_writer_t *xwr = (sqfs_xattr_writer_t *)obj;
+	kv_block_desc_t *blk;
+
+	while (xwr->kv_blocks != NULL) {
+		blk = xwr->kv_blocks;
+		xwr->kv_blocks = xwr->kv_blocks->next;
+		free(blk);
+	}
+
+	free(xwr->kv_pairs);
+	str_table_cleanup(&xwr->values);
+	str_table_cleanup(&xwr->keys);
+	free(xwr);
+}
+
 sqfs_xattr_writer_t *sqfs_xattr_writer_create(void)
 {
 	sqfs_xattr_writer_t *xwr = calloc(1, sizeof(*xwr));
@@ -129,6 +206,8 @@ sqfs_xattr_writer_t *sqfs_xattr_writer_create(void)
 	if (xwr->kv_pairs == NULL)
 		goto fail_pairs;
 
+	((sqfs_object_t *)xwr)->copy = xattr_writer_copy;
+	((sqfs_object_t *)xwr)->destroy = xattr_writer_destroy;
 	return xwr;
 fail_pairs:
 	str_table_cleanup(&xwr->values);
@@ -139,22 +218,6 @@ fail_keys:
 	return NULL;
 }
 
-void sqfs_xattr_writer_destroy(sqfs_xattr_writer_t *xwr)
-{
-	kv_block_desc_t *blk;
-
-	while (xwr->kv_blocks != NULL) {
-		blk = xwr->kv_blocks;
-		xwr->kv_blocks = xwr->kv_blocks->next;
-		free(blk);
-	}
-
-	free(xwr->kv_pairs);
-	str_table_cleanup(&xwr->values);
-	str_table_cleanup(&xwr->keys);
-	free(xwr);
-}
-
 int sqfs_xattr_writer_begin(sqfs_xattr_writer_t *xwr)
 {
 	xwr->kv_start = xwr->num_pairs;
@@ -603,6 +666,6 @@ int sqfs_xattr_writer_flush(sqfs_xattr_writer_t *xwr, sqfs_file_t *file,
 				   locations, count);
 out:
 	free(locations);
-	sqfs_meta_writer_destroy(mw);
+	sqfs_destroy(mw);
 	return err;
 }
