@@ -39,18 +39,16 @@ static int set_working_dir(options_t *opt)
 	return 0;
 }
 
-static int pack_files(sqfs_data_writer_t *data, fstree_t *fs,
-		      data_writer_stats_t *stats, options_t *opt)
+static int pack_files(sqfs_block_processor_t *data, fstree_t *fs,
+		      options_t *opt)
 {
-	sqfs_inode_generic_t *inode;
-	size_t max_blk_count;
+	sqfs_inode_generic_t **inode_ptr;
 	sqfs_u64 filesize;
 	sqfs_file_t *file;
 	tree_node_t *node;
 	const char *path;
 	char *node_path;
 	file_info_t *fi;
-	size_t size;
 	int flags;
 	int ret;
 
@@ -86,50 +84,20 @@ static int pack_files(sqfs_data_writer_t *data, fstree_t *fs,
 			return -1;
 		}
 
-		filesize = file->get_size(file);
-
-		max_blk_count = filesize / opt->cfg.block_size;
-		if (filesize % opt->cfg.block_size)
-			++max_blk_count;
-
-		if (SZ_MUL_OV(sizeof(sqfs_u32), max_blk_count, &size) ||
-		    SZ_ADD_OV(sizeof(*inode), size, &size)) {
-			fputs("creating file inode: too many blocks\n",
-			      stderr);
-			file->destroy(file);
-			free(node_path);
-			return -1;
-		}
-
-		inode = calloc(1, size);
-		if (inode == NULL) {
-			perror("creating file inode");
-			file->destroy(file);
-			free(node_path);
-			return -1;
-		}
-
-		inode->block_sizes = (sqfs_u32 *)inode->extra;
-		inode->base.type = SQFS_INODE_FILE;
-		sqfs_inode_set_file_size(inode, filesize);
-		sqfs_inode_set_frag_location(inode, 0xFFFFFFFF, 0xFFFFFFFF);
-
-		fi->user_ptr = inode;
-
 		flags = 0;
+		filesize = file->get_size(file);
 
 		if (opt->no_tail_packing && filesize > opt->cfg.block_size)
 			flags |= SQFS_BLK_DONT_FRAGMENT;
 
-		ret = write_data_from_file(path, data, inode, file, flags);
-		file->destroy(file);
+		inode_ptr = (sqfs_inode_generic_t **)&fi->user_ptr;
+
+		ret = write_data_from_file(path, data, inode_ptr, file, flags);
+		sqfs_destroy(file);
 		free(node_path);
 
 		if (ret)
 			return -1;
-
-		stats->file_count += 1;
-		stats->bytes_read += filesize;
 	}
 
 	return 0;
@@ -235,7 +203,7 @@ int main(int argc, char **argv)
 	if (fstree_post_process(&sqfs.fs))
 		goto out;
 
-	if (pack_files(sqfs.data, &sqfs.fs, &sqfs.stats, &opt))
+	if (pack_files(sqfs.data, &sqfs.fs, &opt))
 		goto out;
 
 	if (sqfs_writer_finish(&sqfs, &opt.cfg))
