@@ -30,6 +30,7 @@ static struct option long_opts[] = {
 #endif
 	{ "version", no_argument, NULL, 'V' },
 	{ "help", no_argument, NULL, 'h' },
+	{ NULL, 0, NULL, 0 },
 };
 
 static const char *short_opts = "F:D:X:c:b:B:d:j:Q:kxoefqThV"
@@ -185,7 +186,10 @@ void process_command_line(options_t *opt, int argc, char **argv)
 			opt->cfg.comp_id = ret;
 			break;
 		case 'b':
-			opt->cfg.block_size = strtol(optarg, NULL, 0);
+			if (parse_size("Block size", &opt->cfg.block_size,
+				       optarg, 0)) {
+				exit(EXIT_FAILURE);
+			}
 			break;
 		case 'j':
 			opt->cfg.num_jobs = strtol(optarg, NULL, 0);
@@ -194,7 +198,10 @@ void process_command_line(options_t *opt, int argc, char **argv)
 			opt->cfg.max_backlog = strtol(optarg, NULL, 0);
 			break;
 		case 'B':
-			opt->cfg.devblksize = strtol(optarg, NULL, 0);
+			if (parse_size("Device block size",
+				       &opt->cfg.devblksize, optarg, 0)) {
+				exit(EXIT_FAILURE);
+			}
 			if (opt->cfg.devblksize < 1024) {
 				fputs("Device block size must be at "
 				      "least 1024\n", stderr);
