@@ -25,6 +25,7 @@ static struct option long_opts[] = {
 	{ "no-hard-links", no_argument, NULL, 'L' },
 	{ "help", no_argument, NULL, 'h' },
 	{ "version", no_argument, NULL, 'V' },
+	{ NULL, 0, NULL, 0 },
 };
 
 static const char *short_opts = "d:kr:sXLhV";
@@ -297,20 +298,47 @@ fail:
 	return -1;
 }
 
-static char *prepend_prefix(char *name)
+static char *assemble_tar_path(char *name, bool is_dir)
 {
-	size_t len = strlen(root_becomes);
-	char *temp = realloc(name, strlen(name) + len + 2);
+	size_t len, new_len;
+	char *temp;
+	(void)is_dir;
 
+	if (root_becomes == NULL && !is_dir)
+		return name;
+
+	new_len = strlen(name);
+	if (root_becomes != NULL)
+		new_len += strlen(root_becomes) + 1;
+	if (is_dir)
+		new_len += 1;
+
+	temp = realloc(name, new_len + 1);
 	if (temp == NULL) {
 		perror("assembling tar entry filename");
+		free(name);
 		return NULL;
 	}
 
 	name = temp;
-	memmove(name + len + 1, name, strlen(name) + 1);
-	memcpy(name, root_becomes, len);
-	name[len] = '/';
+
+	if (root_becomes != NULL) {
+		len = strlen(root_becomes);
+
+		memmove(name + len + 1, name, strlen(name) + 1);
+		memcpy(name, root_becomes, len);
+		name[len] = '/';
+	}
+
+	if (is_dir) {
+		len = strlen(name);
+
+		if (len == 0 || name[len - 1] != '/') {
+			name[len++] = '/';
+			name[len] = '\0';
+		}
+	}
+
 	return name;
 }
 
@@ -366,11 +394,9 @@ static int write_tree_dfs(const sqfs_tree_node_t *n)
 			}
 		}
 
-		if (root_becomes != NULL) {
-			name = prepend_prefix(name);
-			if (name == NULL)
-				return -1;
-		}
+		name = assemble_tar_path(name, n->children != NULL);
+		if (name == NULL)
+			return -1;
 	}
 
 	inode_stat(n, &sb);
@@ -389,7 +415,7 @@ static int write_tree_dfs(const sqfs_tree_node_t *n)
 		}
 	}
 
-	target = S_ISLNK(sb.st_mode) ? n->inode->slink_target : NULL;
+	target = S_ISLNK(sb.st_mode) ? (char *)n->inode->extra : NULL;
 	ret = write_tar_header(out_file, &sb, name, target, xattr,
 			       record_counter++);
 
@@ -515,44 +541,23 @@ int main(int argc, char **argv)
 		goto out_fd;
 	}
 
-	ret = sqfs_compressor_exists(super.compression_id);
-
-#ifdef WITH_LZO
-	if (super.compression_id == SQFS_COMP_LZO)
-		ret = true;
-#endif
-
-	if (!ret) {
-		fprintf(stderr, "%s: unknown compressor used.\n", filename);
-		goto out_fd;
-	}
-
 	sqfs_compressor_config_init(&cfg, super.compression_id,
 				    super.block_size,
 				    SQFS_COMP_FLAG_UNCOMPRESS);
 
-	cmp = sqfs_compressor_create(&cfg);
+	ret = sqfs_compressor_create(&cfg, &cmp);
 
 #ifdef WITH_LZO
-	if (super.compression_id == SQFS_COMP_LZO && cmp == NULL)
-		cmp = lzo_compressor_create(&cfg);
+	if (super.compression_id == SQFS_COMP_LZO && ret != 0)
+		ret = lzo_compressor_create(&cfg, &cmp);
 #endif
 
-	if (cmp == NULL) {
-		fputs("Error creating compressor.\n", stderr);
+	if (ret != 0) {
+		sqfs_perror(filename, "creating compressor", ret);
 		goto out_fd;
 	}
 
-	if (super.flags & SQFS_FLAG_COMPRESSOR_OPTIONS) {
-		ret = cmp->read_options(cmp, file);
-		if (ret) {
-			sqfs_perror(filename, "reading compressor options",
-				    ret);
-			goto out_cmp;
-		}
-	}
-
-	idtbl = sqfs_id_table_create();
+	idtbl = sqfs_id_table_create(0);
 
 	if (idtbl == NULL) {
 		perror("creating ID table");
@@ -586,14 +591,14 @@ int main(int argc, char **argv)
 	}
 
 	if (!no_xattr && !(super.flags & SQFS_FLAG_NO_XATTRS)) {
-		xr = sqfs_xattr_reader_create(file, &super, cmp);
+		xr = sqfs_xattr_reader_create(0);
 		if (xr == NULL) {
 			sqfs_perror(filename, "creating xattr reader",
 				    SQFS_ERROR_ALLOC);
 			goto out_dr;
 		}
 
-		ret = sqfs_xattr_reader_load_locations(xr);
+		ret = sqfs_xattr_reader_load(xr, &super, file, cmp);
 		if (ret) {
 			sqfs_perror(filename, "loading xattr table", ret);
 			goto out_xr;
@@ -636,12 +641,10 @@ int main(int argc, char **argv)
 		if (sqfs_tree_find_hard_links(root, &links))
 			goto out_tree;
 
-		if (root_becomes != NULL) {
-			for (lnk = links; lnk != NULL; lnk = lnk->next) {
-				lnk->target = prepend_prefix(lnk->target);
-				if (lnk->target == NULL)
-					goto out;
-			}
+		for (lnk = links; lnk != NULL; lnk = lnk->next) {
+			lnk->target = assemble_tar_path(lnk->target, false);
+			if (lnk->target == NULL)
+				goto out;
 		}
 	}
 
@@ -665,17 +668,17 @@ out_tree:
 		sqfs_dir_tree_destroy(root);
 out_xr:
 	if (xr != NULL)
-		sqfs_xattr_reader_destroy(xr);
+		sqfs_destroy(xr);
 out_dr:
-	sqfs_dir_reader_destroy(dr);
+	sqfs_destroy(dr);
 out_data:
-	sqfs_data_reader_destroy(data);
+	sqfs_destroy(data);
 out_id:
-	sqfs_id_table_destroy(idtbl);
+	sqfs_destroy(idtbl);
 out_cmp:
-	cmp->destroy(cmp);
+	sqfs_destroy(cmp);
 out_fd:
-	file->destroy(file);
+	sqfs_destroy(file);
 out_dirs:
 	for (i = 0; i < num_subdirs; ++i)
 		free(subdirs[i]);
